@@ -89,10 +89,6 @@ struct GsdMediaKeysManagerPrivate
         DBusGConnection *connection;
 };
 
-enum {
-        PROP_0,
-};
-
 enum {
         MEDIA_PLAYER_KEY_PRESSED,
         LAST_SIGNAL
@@ -1023,7 +1019,22 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
 void
 gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
 {
+        GsdMediaKeysManagerPrivate *priv = manager->priv;
+
         g_debug ("Stopping media_keys manager");
+
+        if (priv->conf_client) {
+                g_object_unref (priv->conf_client);
+                priv->conf_client = NULL;
+        }
+
+        if (priv->volume) {
+                g_object_unref (priv->volume);
+                priv->volume = NULL;
+        }
+
+        g_slist_free (priv->screens);
+        priv->screens = NULL;
 }
 
 static void
